//---------------------------------------------------------------------------
//
//  Asynch
//
//  Demonstrates asynchronous communication using a loopback from
//  FIO4 to FIO5 on a U3 rev 1.30.  On earlier hardware revisions
//  use SDA and SCL.
//
//  Tested with UD driver V3.03.
//  Tested with U3C firmware V1.08?.
//
//  support@labjack.com
//  July 14, 2009
//----------------------------------------------------------------------

program u3asynch;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  Windows,
  Math,
  LJUDDelphi in '..\..\..\Wrapper\LJUDDelphi.pas';

  //This is our simple error handling procedure that is called after every UD
  //function call.  This function displays the errorcode and string description
  //of the error.  It also has an iteration input that is useful when processing
  //results in a loop (getfirst/getnext).
  procedure ErrorHandler(lngErrorcode: LJ_ERROR; lngIteration: longint);
  var
    err: array[0..254] of AnsiChar;
  begin
    if lngErrorcode <> LJE_NOERROR then
    begin
      ErrorToString(lngErrorcode, @err);
      Writeln('Error number = ' + IntToStr(lngErrorcode));
      Writeln('Error string = ' + string(err));
      Writeln('Iteration = ' + IntToStr(lngIteration));
      Writeln('');
      if lngErrorcode > LJE_MIN_GROUP_ERROR then
      begin
        //Quit if this is a group error.
        Readln;
        Halt;
      end;
    end;
  end;

var
  lngErrorcode: LJ_ERROR;
  lngHandle: LJ_HANDLE;

  numBytes: double;
  Data: array[0..255] of Byte;

begin
  lngHandle := 0;
  //Open the LabJack.
  lngErrorcode := OpenLabJack(LJ_dtU3, LJ_ctUSB, '1', 1, lngHandle);
  ErrorHandler(lngErrorcode, 0);

  //Start by using the pin_configuration_reset IOType so that all
  //pin assignments are in the factory default condition.
  lngErrorcode := ePut(lngHandle, LJ_ioPIN_CONFIGURATION_RESET, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //1 MHz timer clock base.
  lngErrorcode := ePut(lngHandle, LJ_ioPUT_CONFIG, LJ_chTIMER_CLOCK_BASE, LJ_tc1MHZ_DIV, 0);
  ErrorHandler(lngErrorcode, 0);

  //Set clock divisor to 1, so timer clock is 1 MHz.
  lngErrorcode := ePut(lngHandle, LJ_ioPUT_CONFIG, LJ_chTIMER_CLOCK_DIVISOR, LJ_tc1MHZ_DIV, 0);
  ErrorHandler(lngErrorcode, 0);

  //Set timer/counter pin offset to 4. TX and RX appear after any timers and counters on U3
  //hardware rev 1.30.  We have no timers or counters enabled, so TX=FIO4 and RX=FIO5.
  lngErrorcode := ePut(lngHandle, LJ_ioPUT_CONFIG, LJ_chTIMER_COUNTER_PIN_OFFSET, 4, 0);
  ErrorHandler(lngErrorcode, 0);

  //Set data rate for 9600 bps communication.
  lngErrorcode := ePut(lngHandle, LJ_ioPUT_CONFIG, LJ_chASYNCH_BAUDFACTOR, 204, 0);
  ErrorHandler(lngErrorcode, 0);

  //Enable UART.
  lngErrorcode := ePut(lngHandle, LJ_ioASYNCH_COMMUNICATION, LJ_chASYNCH_ENABLE, 1, 0);
  ErrorHandler(lngErrorcode, 0);

  //Transmit 2 bytes.
  numBytes := 2;
  Data[0] := 12;
  Data[1] := 34;
  lngErrorcode := eGet(lngHandle, LJ_ioASYNCH_COMMUNICATION, LJ_chASYNCH_TX, numBytes, @Data);
  ErrorHandler(lngErrorcode, 0);

  //Read 2 bytes.
  numBytes := 9999;  //Dummy values so we can see them change.
  Data[0] := 222;
  Data[1] := 222;
  lngErrorcode := eGet(lngHandle, LJ_ioASYNCH_COMMUNICATION, LJ_chASYNCH_RX, numBytes, @Data);
  ErrorHandler(lngErrorcode, 0);

  //Display the read data.
  Writeln('Pre-read buffer size = ', numBytes);
  Writeln('Read data = ', Data[0], ' , ', Data[1]);

  Readln;
end.
