//---------------------------------------------------------------------------
//
//  u3EI1022
//
//  Demonstrates talking to 1 or 2 EI-1022 probes.
//
//  support@labjack.com
//  May 29, 2007
//----------------------------------------------------------------------

program u3EI1022;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  LJUDDelphi in '..\..\..\Wrapper\LJUDDelphi.pas';


  //This is our simple error handling procedure that is called after every UD
  //function call.  This function displays the errorcode and string description
  //of the error.  It also has an iteration input that is useful when processing
  //results in a loop (getfirst/getnext).
  procedure ErrorHandler(lngErrorcode: LJ_ERROR; lngIteration: longint);
  var
    err: array[0..254] of AnsiChar;
  begin
    if lngErrorcode <> LJE_NOERROR then
    begin
      ErrorToString(lngErrorcode, @err);
      Writeln('Error number = ' + IntToStr(lngErrorcode));
      Writeln('Error string = ' + string(err));
      Writeln('Iteration = ' + IntToStr(lngIteration));
      Writeln('');
      if lngErrorcode > LJE_MIN_GROUP_ERROR then
      begin
        //Quit if this is a group error.
        Readln;
        Halt;
      end;
    end;
  end;

var
  lngErrorcode: LJ_ERROR;
  dblValue: double;
  dblTemp: double;
  lngHandle: LJ_HANDLE;
begin
  dblValue := 0;
  dblTemp := 0;

  //Open the first found LabJack U3.
  lngErrorcode := OpenLabJack(LJ_dtU3, LJ_ctUSB, '1', 1, lngHandle);
  ErrorHandler(lngErrorcode, 0);

  //Start by using the pin_configuration_reset IOType so that all
  //pin assignments are in the factory default condition.
  lngErrorcode := ePut(lngHandle, LJ_ioPIN_CONFIGURATION_RESET, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);


  //Use this code if only a single EI-1022 is connected.
  //  Connections for the probe:
  //  Red (Power)         VS
  //  Black (Ground)      GND
  //  White (Output)      FIO0

  //Configure FIO0 as analog.  That means we will start from channel 0, pass a value
  // of b0000000000000001 or d1 and update 1 bit.
  lngErrorcode := ePut(lngHandle, LJ_ioPUT_ANALOG_ENABLE_PORT, 0, 1, 1);
  ErrorHandler(lngErrorcode, 0);

  //Get AIN0 reading from the EI-1022 probe using the Special 0-3.6 range
  lngErrorcode := eGet(lngHandle, LJ_ioGET_AIN_DIFF, 0, dblValue, 32);
  ErrorHandler(lngErrorcode, 0);

  //Convert analog voltage reading to degrees K (the EI-1022 output is 10 mV per degrees K)
  dblTemp := dblValue * 100.0;

  Writeln('Temp Probe A = ' + FloatToStr(dblTemp) + ' deg K');
  Writeln('Temp Probe A = ' + FloatToStr((dblTemp - 273.15)) + ' deg C');
  Writeln('Temp Probe A = ' + FloatToStr((((dblTemp - 273.15) * 1.8) + 32)) + ' deg F');
  Writeln('');
  //End of single probe code.



  {
  //Use this code if two EI-1022 probes are connected.
  //  Connections for both probes:
  //  Red (Power)         VS
  //  Black (Ground)      GND
  //
  //  Probe A:
  //  White (Output)       FIO0
  //
  //  Probe B:
  //  White (Output)       FIO1


  //Configure FIO0 as analog.  That means we will start from channel 0, pass a value
  //of b0000000000000011 or d3 and update 2 bits.
  lngErrorcode := ePut (lngHandle, LJ_ioPUT_ANALOG_ENABLE_PORT, 0, 3, 2);
  ErrorHandler(lngErrorcode, 0);

  //Request AIN0 and AIN1 readings for both probes using the Special 0-3.6 range.
  lngErrorcode := AddRequest (lngHandle, LJ_ioGET_AIN_DIFF, 0, 0, 32, 0);
  ErrorHandler(lngErrorcode, 0);

  lngErrorcode := AddRequest(lngHandle, LJ_ioGET_AIN_DIFF, 1, 0, 32, 0);
  ErrorHandler(lngErrorcode, 0);

  //Execute the requests.
  lngErrorcode := GoOne (lngHandle);
  ErrorHandler(lngErrorcode, 0);

  //Get AIN0 reading for probe A
  lngErrorcode := GetResult (lngHandle, LJ_ioGET_AIN_DIFF, 0, dblValue);
  ErrorHandler(lngErrorcode, 0);

  //Convert analog voltage reading to degrees K (the EI-1022 output is 10 mV per degrees K)
  dblTemp := dblValue * 100.0;

  Writeln('Temp Probe A = ' + FloatToStr(dblTemp) + ' deg K');
  Writeln('Temp Probe A = ' + FloatToStr((dblTemp-273.15)) + ' deg C');
  Writeln('Temp Probe A = ' + FloatToStr((((dblTemp-273.15)*1.8)+32)) + ' deg F');
  Writeln('');

  //Get AIN1 reading for probe B
  lngErrorcode := GetResult (lngHandle, LJ_ioGET_AIN_DIFF, 1, dblValue);
  ErrorHandler(lngErrorcode, 0);

  //Convert analog voltage reading to degrees K
  dblTemp := dblValue * 100.0;

  Writeln('Temp Probe B = ' + FloatToStr(dblTemp) + ' deg K');
  Writeln('Temp Probe B = ' + FloatToStr((dblTemp-273.15)) + ' deg C');
  Writeln('Temp Probe B = ' + FloatToStr((((dblTemp-273.15)*1.8)+32)) + ' deg F');
  Writeln('');

  //End of dual probe code.
  }

  Writeln('');
  Writeln('Done - Press Enter to exit');
  Readln;

end.

