//---------------------------------------------------------------------------
//
//  u3EI1050
//
//  Demonstrates talking to 1 or 2 EI-1050 probes.
//
//  support@labjack.com
//  June 27, 2013
//----------------------------------------------------------------------

program u3EI1050;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  LJUDDelphi in '..\..\..\Wrapper\LJUDDelphi.pas';

  //This is our simple error handling procedure that is called after every UD
  //function call.  This function displays the errorcode and string description
  //of the error.  It also has an iteration input that is useful when processing
  //results in a loop (getfirst/getnext).
  procedure ErrorHandler(lngErrorcode: LJ_ERROR; lngIteration: longint);
  var
    err: array[0..254] of AnsiChar;
  begin
    if lngErrorcode <> LJE_NOERROR then
    begin
      ErrorToString(lngErrorcode, @err);
      Writeln('Error number = ' + IntToStr(lngErrorcode));
      Writeln('Error string = ' + string(err));
      Writeln('Iteration = ' + IntToStr(lngIteration));
      Writeln('');
      if lngErrorcode > LJE_MIN_GROUP_ERROR then
      begin
        //Quit if this is a group error.
        Readln;
        Halt;
      end;
    end;
  end;

var
  lngErrorcode: LJ_ERROR;
  dblValue: double;
  lngHandle: LJ_HANDLE;
begin
  dblValue := 0;

  //Open the first found LabJack U3.
  lngErrorcode := OpenLabJack(LJ_dtU3, LJ_ctUSB, '1', 1, lngHandle);
  ErrorHandler(lngErrorcode, 0);

  //Start by using the pin_configuration_reset IOType so that all
  //pin assignments are in the factory default condition.
  lngErrorcode := ePut(lngHandle, LJ_ioPIN_CONFIGURATION_RESET, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);


  //Set the Data line to FIO4, which is the default anyway.
  lngErrorcode := ePut(lngHandle, LJ_ioPUT_CONFIG, LJ_chSHT_DATA_CHANNEL, 4, 0);
  ErrorHandler(lngErrorcode, 0);

  //Set the Clock line to FIO5, which is the default anyway.
  lngErrorcode := ePut(lngHandle, LJ_ioPUT_CONFIG, LJ_chSHT_CLOCK_CHANNEL, 5, 0);
  ErrorHandler(lngErrorcode, 0);

  //Set FIO6 to output-high to provide power to the EI-1050s.
  lngErrorcode := ePut(lngHandle, LJ_ioPUT_DIGITAL_BIT, 6, 1, 0);
  ErrorHandler(lngErrorcode, 0);



  //Use this code if only a single EI-1050 is connected.
  //  Connections for the probe:
  //  Red (Power)         FIO6
  //  Black (Ground)      GND
  //  Green (Data)        FIO4
  //  White (Clock)       FIO5
  //  Brown (Enable)      FIO6

  //Now, an add/go/get block to get the temp & humidity at the same time.
  //Request a temperature reading from the EI-1050.
  lngErrorcode := AddRequest(lngHandle, LJ_ioSHT_GET_READING, LJ_chSHT_TEMP, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Request a humidity reading from the EI-1050.
  lngErrorcode := AddRequest(lngHandle, LJ_ioSHT_GET_READING, LJ_chSHT_RH, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Execute the requests.  Will take about 0.5 seconds with a USB high-high
  //or Ethernet connection, and about 1.5 seconds with a normal USB connection.
  lngErrorcode := GoOne(lngHandle);
  ErrorHandler(lngErrorcode, 0);

  //Get the temperature reading.
  lngErrorcode := GetResult(lngHandle, LJ_ioSHT_GET_READING, LJ_chSHT_TEMP, dblValue);
  ErrorHandler(lngErrorcode, 0);
  Writeln('Temp Probe A = ' + FloatToStr(dblValue) + ' deg K');
  Writeln('Temp Probe A = ' + FloatToStr((dblValue - 273.15)) + ' deg C');
  Writeln('Temp Probe A = ' + FloatToStr((((dblValue - 273.15) * 1.8) + 32)) + ' deg F');

  //Get the humidity reading.
  lngErrorcode := GetResult(lngHandle, LJ_ioSHT_GET_READING, LJ_chSHT_RH, dblValue);
  ErrorHandler(lngErrorcode, 0);
  Writeln('RH Probe A = ' + FloatToStr(dblValue) + ' percent');
  Writeln('');
  //End of single probe code.


  {
  //Use this code if two EI-1050 probes are connected.
  //  Connections for both probes:
  //  Red (Power)         FIO6
  //  Black (Ground)      GND
  //  Green (Data)        FIO4
  //  White (Clock)       FIO5
  //
  //  Probe A:
  //  Brown (Enable)    FIO7
  //
  //  Probe B:
  //  Brown (Enable)    DAC0

  //Set FIO7 to output-low to disable probe A.
  lngErrorcode := ePut(lngHandle, LJ_ioPUT_DIGITAL_BIT, 7, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Set DAC0 to 0 volts to disable probe B.
  lngErrorcode := ePut(lngHandle, LJ_ioPUT_DAC, 0, 0.0, 0);
  ErrorHandler(lngErrorcode, 0);


  //Set FIO7 to output-high to enable probe A.
  lngErrorcode := ePut(lngHandle, LJ_ioPUT_DIGITAL_BIT, 7, 1, 0);
  ErrorHandler(lngErrorcode, 0);

  //Now, an add/go/get block to get the temp & humidity at the same time.
  //Request a temperature reading from the EI-1050.
  lngErrorcode := AddRequest(lngHandle, LJ_ioSHT_GET_READING, LJ_chSHT_TEMP, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Request a humidity reading from the EI-1050.
  lngErrorcode := AddRequest(lngHandle, LJ_ioSHT_GET_READING, LJ_chSHT_RH, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Execute the requests.  Will take about 0.5 seconds with a USB high-high
  //or Ethernet connection, and about 1.5 seconds with a normal USB connection.
  lngErrorcode := GoOne(lngHandle);
  ErrorHandler(lngErrorcode, 0);

  //Get the temperature reading.
  lngErrorcode := GetResult(lngHandle, LJ_ioSHT_GET_READING, LJ_chSHT_TEMP, dblValue);
  ErrorHandler(lngErrorcode, 0);
  Writeln('Temp Probe A = ' + FloatToStr(dblValue) + ' deg K');
  Writeln('Temp Probe A = ' + FloatToStr((dblValue-273.15)) + ' deg C');
  Writeln('Temp Probe A = ' + FloatToStr((((dblValue-273.15)*1.8)+32)) + ' deg F');

  //Get the humidity reading.
  lngErrorcode := GetResult(lngHandle, LJ_ioSHT_GET_READING, LJ_chSHT_RH, dblValue);
  ErrorHandler(lngErrorcode, 0);
  Writeln('RH Probe A = ' + FloatToStr(dblValue) + ' percent');
  Writeln('');

  //Set FIO7 to output-low to disable probe A.
  lngErrorcode := ePut(lngHandle, LJ_ioPUT_DIGITAL_BIT, 7, 0, 0);
  ErrorHandler(lngErrorcode, 0);


  //Set DAC0 to 3.3 volts to enable probe B.
  lngErrorcode := ePut(lngHandle, LJ_ioPUT_DAC, 0, 3.3, 0);
  ErrorHandler(lngErrorcode, 0);

  //Since the DACs on the U3 are slower than the communication speed,
  //we put a delay here to make sure the DAC has time to rise to 3.3 volts
  //before communicating with the EI-1050.
  Sleep(30);  //Wait 30 ms.

  //Now, an add/go/get block to get the temp & humidity at the same time.
  //Request a temperature reading from the EI-1050.
  lngErrorcode := AddRequest(lngHandle, LJ_ioSHT_GET_READING, LJ_chSHT_TEMP, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Request a humidity reading from the EI-1050.
  lngErrorcode := AddRequest(lngHandle, LJ_ioSHT_GET_READING, LJ_chSHT_RH, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Execute the requests.  Will take about 0.5 seconds with a USB high-high
  //or Ethernet connection, and about 1.5 seconds with a normal USB connection.
  lngErrorcode := GoOne(lngHandle);
  ErrorHandler(lngErrorcode, 0);

  //Get the temperature reading.
  lngErrorcode := GetResult(lngHandle, LJ_ioSHT_GET_READING, LJ_chSHT_TEMP, dblValue);
  ErrorHandler(lngErrorcode, 0);
  Writeln('Temp Probe B = ' + FloatToStr(dblValue) + ' deg K');
  Writeln('Temp Probe B = ' + FloatToStr((dblValue-273.15)) + ' deg C');
  Writeln('Temp Probe B = ' + FloatToStr((((dblValue-273.15)*1.8)+32)) + ' deg F');

  //Get the humidity reading.
  lngErrorcode := GetResult(lngHandle, LJ_ioSHT_GET_READING, LJ_chSHT_RH, dblValue);
  ErrorHandler(lngErrorcode, 0);
  Writeln('RH Probe B = ' + FloatToStr(dblValue) + ' percent');
  Writeln('');

  //Set DAC0 to 0 volts to disable probe B.
  lngErrorcode := ePut(lngHandle, LJ_ioPUT_DAC, 0, 0.0, 0);
  ErrorHandler(lngErrorcode, 0);

  //If we were going to loop and talk to probe A next, we would
  //want a delay here to make sure the DAC falls to 0 volts
  //before enabling probe A.
  Sleep(30);  //Wait 30 ms.

  //End of dual probe code.
  }

  Writeln('');
  Writeln('Done - Press Enter to exit');
  Readln;

end.

