//---------------------------------------------------------------------------
//
//  U3timercounter
//
//  Basic U3 example does a PWM output and a counter input, using AddGoGet
//  method.
//
//  support@labjack.com
//  Jul 21, 2008
//----------------------------------------------------------------------


program timercounter;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  Windows,
  Math,
  LJUDDelphi in '..\..\..\Wrapper\LJUDDelphi.pas';

  //This is our simple error handling procedure that is called after every UD
  //function call.  This function displays the errorcode and string description
  //of the error.  It also has an iteration input that is useful when processing
  //results in a loop (getfirst/getnext).
  procedure ErrorHandler(lngErrorcode: LJ_ERROR; lngIteration: longint);
  var
    err: array[0..254] of AnsiChar;
  begin
    if lngErrorcode <> LJE_NOERROR then
    begin
      ErrorToString(lngErrorcode, @err);
      Writeln('Error number = ' + IntToStr(lngErrorcode));
      Writeln('Error string = ' + string(err));
      Writeln('Iteration = ' + IntToStr(lngIteration));
      Writeln('');
      if lngErrorcode > LJE_MIN_GROUP_ERROR then
      begin
        //Quit if this is a group error.
        Readln;
        Halt;
      end;
    end;
  end;

var
  lngErrorcode: LJ_ERROR;
  lngGetNextIteration: longint;
  lngIOType, lngChannel: longint;
  dblValue: double;
  lngHandle: LJ_HANDLE;
  dummyInt: longint;
  dummyDouble: double;

begin
  lngIOType := 0;
  lngChannel := 0;
  dblValue := 0;
  lngHandle := 0;

  //Open the first found LabJack U3.
  lngErrorcode := OpenLabJack(LJ_dtU3, LJ_ctUSB, '1', 1, lngHandle);
  ErrorHandler(lngErrorcode, 0);

  //Start by using the pin_configuration_reset IOType so that all
  //pin assignments are in the factory default condition.
  lngErrorcode := ePut(lngHandle, LJ_ioPIN_CONFIGURATION_RESET, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);


  //First requests to configure the timer and counter.  These will be
  //done with and add/go/get block.

  //Set the timer/counter pin offset to 4, which will put the first
  //timer/counter on FIO4.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chTIMER_COUNTER_PIN_OFFSET, 4, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Use the 48 MHz timer clock base with divider.  Since we are using clock with divisor
  //support, Counter0 is not available.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chTIMER_CLOCK_BASE, LJ_tc48MHZ_DIV, 0, 0);
  //lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chTIMER_CLOCK_BASE, LJ_tc24MHZ_DIV, 0, 0);  //Use this line instead for hardware rev 1.20.
  ErrorHandler(lngErrorcode, 0);

  //Set the divisor to 48 so the actual timer clock is 1 MHz.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chTIMER_CLOCK_DIVISOR, 48, 0, 0);
  //lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chTIMER_CLOCK_DIVISOR, 24, 0, 0);  //Use this line instead for hardware rev 1.20.
  ErrorHandler(lngErrorcode, 0);

  //Enable 1 timer.  It will use FIO4.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chNUMBER_TIMERS_ENABLED, 1, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Configure Timer0 as 8-bit PWM.  Frequency will be 1M/256 = 3906 Hz.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_TIMER_MODE, 0, LJ_tmPWM8, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Set the PWM duty cycle to 50%.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_TIMER_VALUE, 0, 32768, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Enable Counter1.  It will use FIO5 since 1 timer is enabled.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_COUNTER_ENABLE, 1, 1, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Execute the requests.
  lngErrorcode := GoOne(lngHandle);
  ErrorHandler(lngErrorcode, 0);

  //Get all the results just to check for errors.
  lngErrorcode := GetFirstResult(lngHandle, lngIOType, lngChannel, dblValue, dummyInt, dummyDouble);
  ErrorHandler(lngErrorcode, 0);
  lngGetNextIteration := 0;
  while lngErrorcode < LJE_MIN_GROUP_ERROR do
  begin
    lngErrorcode := GetNextResult(lngHandle, lngIOType, lngChannel, dblValue, dummyInt, dummyDouble);
    if not lngErrorcode = LJE_NO_MORE_DATA_AVAILABLE then
    begin
      ErrorHandler(lngErrorcode, lngGetNextIteration);
    end;

    lngGetNextIteration := lngGetNextIteration + 1;
  end;


  //Wait 1 second.
  Sleep(1000);

  //Request a read from the counter.
  lngErrorcode := eGet(lngHandle, LJ_ioGET_COUNTER, 1, dblValue, 0);
  ErrorHandler(lngErrorcode, 0);

  //This should read roughly 4k counts if FIO4 is shorted to FIO5.
  Writeln('Counter = ', dblValue);

  //Wait 1 second.
  Sleep(1000);

  //Request a read from the counter.
  lngErrorcode := eGet(lngHandle, LJ_ioGET_COUNTER, 1, dblValue, 0);
  ErrorHandler(lngErrorcode, 0);

  //This should read about 3906 counts more than the previous read.
  Writeln('Counter = ', dblValue);

  //Reset all pin assignments to factory default condition.
  lngErrorcode := ePut(lngHandle, LJ_ioPIN_CONFIGURATION_RESET, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //The PWM output sets FIO4 to output, so we do a read here to set
  //it to input.
  lngErrorcode := eGet(lngHandle, LJ_ioGET_DIGITAL_BIT, 4, dblValue, 0);
  ErrorHandler(lngErrorcode, 0);

  readln;
end.
