//---------------------------------------------------------------------------
//
//  u3efunctions.dpr
//
//  Demonstrates the UD E-functions with the LabJack U3.
//
//  support@labjack.com
//  June 27, 2013
//----------------------------------------------------------------------

program u3efunctions;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  LJUDDelphi in '..\..\..\Wrapper\LJUDDelphi.pas';

  //This is our simple error handling procedure that is called after every UD
  //function call.  This function displays the errorcode and string description
  //of the error.  It also has an iteration input that is useful when processing
  //results in a loop (getfirst/getnext).
  procedure ErrorHandler(lngErrorcode: LJ_ERROR; lngIteration: longint);
  var
    err: array[0..254] of AnsiChar;
  begin
    if lngErrorcode <> LJE_NOERROR then
    begin
      ErrorToString(lngErrorcode, @err);
      Writeln('Error number = ' + IntToStr(lngErrorcode));
      Writeln('Error string = ' + string(err));
      Writeln('Iteration = ' + IntToStr(lngIteration));
      Writeln('');
      if lngErrorcode > LJE_MIN_GROUP_ERROR then
      begin
        //Quit if this is a group error.
        Readln;
        Halt;
      end;
    end;
  end;

var
  lngErrorcode: LJ_ERROR;
  dblValue: double;
  lngValue: longint;
  lngHandle: LJ_HANDLE;
  lngBinary: longint;
  alngEnableTimers, alngEnableCounters: array[0..1] of longint;
  lngTCPinOffset, lngTimerClockBaseIndex, lngTimerClockDivisor: longint;
  alngTimerModes: array[0..1] of longint;
  adblTimerValues: array[0..1] of double;
  alngReadTimers, alngUpdateResetTimers, alngReadCounters,
  alngResetCounters: array[0..1] of longint;
  adblCounterValues: array[0..1] of double;
  highTime, lowTime, dutyCycle: double;
begin
  dblValue := 0;
  lngValue := 0;
  lngHandle := 0;
  adblCounterValues[0] := 0;
  adblCounterValues[1] := 0;

  //Open the first found LabJack U3.
  lngErrorcode := OpenLabJack(LJ_dtU3, LJ_ctUSB, '1', 1, lngHandle);
  ErrorHandler(lngErrorcode, 0);

  //Start by using the pin_configuration_reset IOType so that all
  //pin assignments are in the factory default condition.
  lngErrorcode := ePut(lngHandle, LJ_ioPIN_CONFIGURATION_RESET, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Take a measurement from AIN3.  `
  lngBinary := 0;
  lngErrorcode := eAIN(lngHandle, 3, 0, dblValue, -1, -1, -1, lngBinary, 0, 0);
  ErrorHandler(lngErrorcode, 0);
  Writeln('AIN3 = ' + FloatToStr(dblValue));

  //Set DAC0 to 3.0 volts.
  dblValue := 3.0;
  lngBinary := 0;
  lngErrorcode := eDAC(lngHandle, 0, dblValue, lngBinary, 0, 0);
  ErrorHandler(lngErrorcode, 0);
  Writeln('DAC0 set to ' + FloatToStr(dblValue) + ' volts');

  //Read state of FIO4.
  lngErrorcode := eDI(lngHandle, 4, lngValue);
  ErrorHandler(lngErrorcode, 0);
  Writeln('FIO4 = ' + FloatToStr(lngValue));

  //Set the state of FIO7.
  lngValue := 0;
  lngErrorcode := eDO(lngHandle, 7, lngValue);
  ErrorHandler(lngErrorcode, 0);
  Writeln('FIO7 set to = ' + FloatToStr(lngValue));
  Writeln('');

  //Timers and Counters example.
  //First, a call to eTCConfig.  Fill the arrays with the desired values, then
  //make the call.
  alngEnableTimers[0] := 1; //Enable Timer0 (uses FIO4).
  alngEnableTimers[1] := 1; //Enable Timer1 (uses FIO5).
  alngEnableCounters[0] := 0; //Disable Counter0.
  alngEnableCounters[1] := 1; //Enable Counter1 (uses FIO6).
  lngTCPinOffset := 4;  //Offset is zero, so timers/counters start at FIO4.
  lngTimerClockBaseIndex := LJ_tc48MHZ_DIV;
  //lngTimerClockBaseIndex := LJ_tc24MHZ_DIV;  //Use this line instead for hardware rev 1.20.
  //Base clock is 24 MHz with divisor support, so Counter0 is not available.
  lngTimerClockDivisor := 48; //Thus timer clock is 1 MHz.
  //lngTimerClockDivisor := 24;  //Use this line instead for hardware rev 1.20.
  alngTimerModes[0] := LJ_tmPWM8; //Timer0 is 8-bit PWM output.  Frequency is 1M/256 = 3906 Hz.
  alngTimerModes[1] := LJ_tmDUTYCYCLE; //Timer1 is duty cyle input.
  adblTimerValues[0] := 16384; //Set PWM8 duty-cycle to 75%.
  adblTimerValues[1] := 0;
  lngErrorcode := eTCConfig(lngHandle, @alngEnableTimers, @alngEnableCounters, lngTCPinOffset, lngTimerClockBaseIndex, lngTimerClockDivisor, @alngTimerModes, @adblTimerValues, 0, 0);
  ErrorHandler(lngErrorcode, 0);
  Writeln('Timers and Counters enabled.');
  Writeln('');

  Sleep(1000); //Wait 1 second.

  //Now, a call to eTCValues.
  alngReadTimers[0] := 0; //Don't read Timer0 (output timer).
  alngReadTimers[1] := 1; //Read Timer1;
  alngUpdateResetTimers[0] := 1; //Update Timer0;
  alngUpdateResetTimers[1] := 1; //Reset Timer1;
  alngReadCounters[0] := 0;
  alngReadCounters[1] := 1; //Read Counter1;
  alngResetCounters[0] := 0;
  alngResetCounters[1] := 1; //Reset Counter1.
  adblTimerValues[0] := 32768; //Change Timer0 duty-cycle to 50%.
  adblTimerValues[1] := 0;
  lngErrorcode := eTCValues(lngHandle, @alngReadTimers, @alngUpdateResetTimers, @alngReadCounters, @alngResetCounters, @adblTimerValues, @adblCounterValues, 0, 0);
  ErrorHandler(lngErrorcode, 0);
  Writeln('Timer1 value = ' + FloatToStr(adblTimerValues[1]));
  Writeln('Counter1 value = ' + FloatToStr(adblCounterValues[1]));

  //Convert Timer1 value to duty-cycle percentage.
  //High time is LSW
  highTime := trunc(adblTimerValues[1]) mod 65536;
  //Low time is MSW
  lowTime := trunc(adblTimerValues[1]) / 65536;
  //Calculate the duty cycle percentage.
  dutyCycle := 100 * highTime / (highTime + lowTime);
  Writeln('');
  Writeln('High clicks Timer1 = ' + FloatToStr(highTime));
  Writeln('Low clicks Timer1 = ' + FloatToStr(lowTime));
  Writeln('Duty cycle Timer1 = ' + FloatToStr(dutyCycle));


  //Disable all timers and counters.
  alngEnableTimers[0] := 0;
  alngEnableTimers[1] := 0;
  alngEnableCounters[0] := 0;
  alngEnableCounters[1] := 0;
  lngErrorcode := eTCConfig(lngHandle, @alngEnableTimers, @alngEnableCounters, 4, lngTimerClockBaseIndex, lngTimerClockDivisor, @alngTimerModes, @adblTimerValues, 0, 0);
  ErrorHandler(lngErrorcode, 0);
  Writeln('');
  Writeln('Done - Press Enter to exit');
  Readln;
end.
