//---------------------------------------------------------------------------
//
//  U6twounits.cpp
//
//  Simple example demonstrates communication with 2 U6s.
//
//  support@labjack.com
//  Feb 26, 2007
//----------------------------------------------------------------------


program twounits;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  Windows,
  Math,
  LJUDDelphi in '..\..\..\Wrapper\LJUDDelphi.pas';

  //This is our simple error handling procedure that is called after every UD
  //function call.  This function displays the errorcode and string description
  //of the error.  It also has an iteration input that is useful when processing
  //results in a loop (getfirst/getnext).
  procedure ErrorHandler(lngErrorcode: LJ_ERROR; lngIteration: longint);
  var
    err: array[0..254] of AnsiChar;
  begin
    if lngErrorcode <> LJE_NOERROR then
    begin
      ErrorToString(lngErrorcode, @err);
      Writeln('Error number = ' + IntToStr(lngErrorcode));
      Writeln('Error string = ' + string(err));
      Writeln('Iteration = ' + IntToStr(lngIteration));
      Writeln('');
      if lngErrorcode > LJE_MIN_GROUP_ERROR then
      begin
        //Quit if this is a group error.
        Readln;
        Halt;
      end;
    end;
  end;

var
  lngErrorcode: LJ_ERROR;
  lngGetNextIteration: longint;
  dblDriverVersion: double;
  lngIOType, lngChannel: longint;
  dblValue: double;
  Value22, Value32, Value42: double;
  Value23, Value33, Value43: double;
  lngHandle2, lngHandle3: LJ_HANDLE;
  ch: char;
  dummyInt: longint;
  dummyDouble: double;

begin
  lngIOType := 0;
  lngChannel := 0;
  dblValue := 0;
  Value22 := 9999;
  Value32 := 9999;
  Value42 := 9999;
  Value23 := 9999;
  Value33 := 9999;
  Value43 := 9999;
  lngHandle2 := 0;
  lngHandle3 := 0;

  //Read and display the UD version.
  dblDriverVersion := GetDriverVersion();
  Writeln('UD Driver Version = ', dblDriverVersion);


  //Open the U6 with local ID 2.
  lngErrorcode := OpenLabJack(LJ_dtU6, LJ_ctUSB, '2', 0, lngHandle2);
  ErrorHandler(lngErrorcode, 0);

  //Open the U6 with local ID 3.
  lngErrorcode := OpenLabJack(LJ_dtU6, LJ_ctUSB, '3', 0, lngHandle3);
  ErrorHandler(lngErrorcode, 0);

  //Start by using the pin_configuration_reset IOType so that all
  //pin assignments are in the factory default condition.
  lngErrorcode := ePut(lngHandle2, LJ_ioPIN_CONFIGURATION_RESET, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);
  lngErrorcode := ePut(lngHandle3, LJ_ioPIN_CONFIGURATION_RESET, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //The following commands will use the add-go-get method to group
  //multiple requests into a single low-level function.

  //Request a single-ended reading from AIN2.
  lngErrorcode := AddRequest(lngHandle2, LJ_ioGET_AIN, 2, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);
  lngErrorcode := AddRequest(lngHandle3, LJ_ioGET_AIN, 2, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Request a single-ended reading from AIN3.
  lngErrorcode := AddRequest(lngHandle2, LJ_ioGET_AIN, 3, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);
  lngErrorcode := AddRequest(lngHandle3, LJ_ioGET_AIN, 3, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Request a reading from AIN4 using the Special 0-3.6 range.
  lngErrorcode := AddRequest(lngHandle2, LJ_ioGET_AIN_DIFF, 4, 0, 32, 0);
  ErrorHandler(lngErrorcode, 0);
  lngErrorcode := AddRequest(lngHandle3, LJ_ioGET_AIN_DIFF, 4, 0, 32, 0);
  ErrorHandler(lngErrorcode, 0);


  while True do
  begin

    //Execute all requests on all open LabJacks.
    lngErrorcode := Go();
    ErrorHandler(lngErrorcode, 0);


    //Get all the results for unit 2.  The input measurement results are stored.
    lngErrorcode := GetFirstResult(lngHandle2, lngIOType, lngChannel, dblValue, dummyInt, dummyDouble);
    ErrorHandler(lngErrorcode, 0);

    lngGetNextIteration := 0;
    while lngErrorcode < LJE_MIN_GROUP_ERROR do
    begin
      if lngIOType = LJ_ioGET_AIN then
      begin
        if lngChannel = 2 then
          Value22 := dblValue;

        if lngChannel = 3 then
          Value32 := dblValue;

      end;
      if lngIOType = LJ_ioGET_AIN_DIFF then
        Value42 := dblValue;

      lngErrorcode := GetNextResult(lngHandle2, lngIOType, lngChannel, dblValue, dummyInt, dummyDouble);
      if not lngErrorcode = LJE_NO_MORE_DATA_AVAILABLE then
        ErrorHandler(lngErrorcode, lngGetNextIteration);

      lngGetNextIteration := lngGetNextIteration + 1;

    end;


    //Get all the results for unit 3.  The input measurement results are stored.
    lngErrorcode := GetFirstResult(lngHandle3, lngIOType, lngChannel, dblValue, dummyInt, dummyDouble);
    ErrorHandler(lngErrorcode, 0);

    lngGetNextIteration := 0;
    while lngErrorcode < LJE_MIN_GROUP_ERROR do
    begin
      if lngIOType = LJ_ioGET_AIN then
      begin
        if lngChannel = 2 then
          Value23 := dblValue;

        if lngChannel = 3 then
          Value33 := dblValue;

      end;
      if lngIOType = LJ_ioGET_AIN_DIFF then
        Value43 := dblValue;

      lngErrorcode := GetNextResult(lngHandle2, lngIOType, lngChannel, dblValue, dummyInt, dummyDouble);
      if not lngErrorcode = LJE_NO_MORE_DATA_AVAILABLE then
        ErrorHandler(lngErrorcode, lngGetNextIteration);

      lngGetNextIteration := lngGetNextIteration + 1;

    end;

    Writeln('AIN2 (Unit 2) = ', Value22);
    Writeln('AIN2 (Unit 3) = ', Value23);
    Writeln('AIN3 (Unit 2) = ', Value32);
    Writeln('AIN3 (Unit 3) = ', Value33);
    Writeln('AIN4 (Unit 2) = ', Value42);
    Writeln('AIN4 (Unit 3) = ', Value43);

    Writeln('Press Enter to go again or (q) to quit');
    Readln(ch);
    if ch = 'q' then
      Halt;
  end;
end.
