//---------------------------------------------------------------------------
//
//  u6efunctions.dpr
//
//  Demonstrates the UD E-functions with the LabJack U6.
//
//  support@labjack.com
//  July 14, 2009
//----------------------------------------------------------------------

program u3efunctions;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  LJUDDelphi in '..\..\..\Wrapper\LJUDDelphi.pas';

  //This is our simple error handling procedure that is called after every UD
  //function call.  This function displays the errorcode and string description
  //of the error.  It also has an iteration input that is useful when processing
  //results in a loop (getfirst/getnext).
  procedure ErrorHandler(lngErrorcode: LJ_ERROR; lngIteration: longint);
  var
    err: array[0..254] of AnsiChar;
  begin
    if lngErrorcode <> LJE_NOERROR then
    begin
      ErrorToString(lngErrorcode, @err);
      Writeln('Error number = ' + IntToStr(lngErrorcode));
      Writeln('Error string = ' + string(err));
      Writeln('Iteration = ' + IntToStr(lngIteration));
      Writeln('');
      if lngErrorcode > LJE_MIN_GROUP_ERROR then
      begin
        //Quit if this is a group error.
        Readln;
        Halt;
      end;
    end;
  end;

var
  lngErrorcode: LJ_ERROR;
  dblValue: double;
  lngValue: longint;
  lngHandle: LJ_HANDLE;
  lngRange, lngResolution, lngBinary: longint;
  alngEnableTimers: array[0..5] of longint;
  alngEnableCounters: array[0..1] of longint;
  lngTimerClockBaseIndex, lngTimerClockDivisor: longint;
  alngTimerModes: array[0..5] of longint;
  adblTimerValues: array[0..5] of double;
  alngReadTimers, alngUpdateResetTimers: array[0..5] of longint;
  alngReadCounters, alngResetCounters: array[0..1] of longint;
  adblCounterValues: array[0..1] of double;
  highTime, lowTime, dutyCycle: double;
begin
  dblValue := 0;
  lngValue := 0;
  lngHandle := 0;
  adblCounterValues[0] := 0;
  adblCounterValues[1] := 0;

  //Open the first found LabJack U6.
  lngErrorcode := OpenLabJack(LJ_dtU6, LJ_ctUSB, '1', 1, lngHandle);
  ErrorHandler(lngErrorcode, 0);

  //Take a measurement from AIN3.
  lngRange := LJ_rgBIP5V;
  lngResolution := 17;
  lngBinary := 0;
  lngErrorcode := eAIN(lngHandle, 3, 0, dblValue, lngRange, lngResolution, -1, lngBinary, 0, 0);
  ErrorHandler(lngErrorcode, 0);
  Writeln('AIN3 = ' + FloatToStr(dblValue));

  //Set DAC0 to 3.0 volts.
  dblValue := 3.0;
  lngBinary := 0;
  lngErrorcode := eDAC(lngHandle, 0, dblValue, lngBinary, 0, 0);
  ErrorHandler(lngErrorcode, 0);
  Writeln('DAC0 set to ' + FloatToStr(dblValue) + ' volts');

  //Read state of FIO2.
  lngErrorcode := eDI(lngHandle, 2, lngValue);
  ErrorHandler(lngErrorcode, 0);
  Writeln('FIO2 = ' + FloatToStr(lngValue));

  //Set the state of FIO3.
  lngValue := 0;
  lngErrorcode := eDO(lngHandle, 3, lngValue);
  ErrorHandler(lngErrorcode, 0);
  Writeln('FIO3 set to = ' + FloatToStr(lngValue));
  Writeln('');

  Writeln('');
  Writeln('Done - Press Enter to exit');
  Readln;
end.
