//---------------------------------------------------------------------------
//
//  ue9memory
//
//  Demonstrates writing and reading to the user and cal memory on the UE9.
//
//  support@labjack.com
//  Jul 14, 2009
//----------------------------------------------------------------------

program ue9memory;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  Windows,
  Math,
  LJUDDelphi in '..\..\..\Wrapper\LJUDDelphi.pas';

  //This is our simple error handling procedure that is called after every UD
  //function call.  This function displays the errorcode and string description
  //of the error.  It also has an iteration input that is useful when processing
  //results in a loop (getfirst/getnext).
  procedure ErrorHandler(lngErrorcode: LJ_ERROR; lngIteration: longint);
  var
    err: array[0..254] of AnsiChar;
  begin
    if lngErrorcode <> LJE_NOERROR then
    begin
      ErrorToString(lngErrorcode, @err);
      Writeln('Error number = ' + IntToStr(lngErrorcode));
      Writeln('Error string = ' + string(err));
      Writeln('Iteration = ' + IntToStr(lngIteration));
      Writeln('');
      if lngErrorcode > LJE_MIN_GROUP_ERROR then
      begin
        //Quit if this is a group error.
        Readln;
        Halt;
      end;
    end;
  end;

var
  lngErrorcode: LJ_ERROR;
  lngHandle: LJ_HANDLE;
  i: longint;
  achrUserMem: array[0..1023] of byte;
  adblCalMem: array[0..127] of double;
  dummyInt: longint;
  dummyDouble: double;

begin
  Randomize;
  lngHandle := 0;
  i := 0;

  //Open the first found LabJack UE9.
  lngErrorcode := OpenLabJack(LJ_dtUE9, LJ_ctUSB, '1', 1, lngHandle);
  //lngErrorcode := OpenLabJack (LJ_dtUE9, LJ_ctETHERNET, '192.168.1.209', 0, lngHandle);
  ErrorHandler(lngErrorcode, 0);


  //First a user memory example.  We will read the memory, update a few elements,
  //and write the memory.  The entire memory area is read and written each time.
  //The user memory is just stored as bytes, so almost any information can be
  //put in there such as integers, doubles, or strings.

  //Read the user memory.
  lngErrorcode := eGet(lngHandle, LJ_ioGET_CONFIG, LJ_chUSER_MEM, dummyDouble, @achrUserMem[0]);
  ErrorHandler(lngErrorcode, 0);
  //Display the first 4 elements.
  Writeln('Read User Mem = ', achrUserMem[0], ', ', achrUserMem[1], ', ', achrUserMem[2], ', ', achrUserMem[3]);
  //Create 4 new pseudo-random numbers to write.  We will update the first
  //4 elements of user memory, but the rest will be unchanged.
  for i := 0 to 3 do
  begin
    achrUserMem[i] := Trunc(255 * (Random(100) / 100));
  end;
  Writeln('Read Write User Mem = ', achrUserMem[0], ', ', achrUserMem[1], ', ', achrUserMem[2], ', ', achrUserMem[3]);
  lngErrorcode := ePut(lngHandle, LJ_ioPUT_CONFIG, LJ_chUSER_MEM, dummyInt, @achrUserMem[0]);
  ErrorHandler(lngErrorcode, 0);
  //Re-read the user memory.
  lngErrorcode := eGet(lngHandle, LJ_ioGET_CONFIG, LJ_chUSER_MEM, dummyDouble, @achrUserMem[0]);
  ErrorHandler(lngErrorcode, 0);
  //Display the first 4 elements.
  Writeln('Read User Mem = ', achrUserMem[0], ', ', achrUserMem[1], ', ', achrUserMem[2], ', ', achrUserMem[3]);


  {
  //Now a cal constants example.  The calibration memory is passed as doubles.
  //The memory area consists of 8 blocks (0-7) of 16 doubles each, for a total
  //of 128 elements.  As of this writing, block 7 is not used, so we will
  //use the last 4 elements of block 7 for testing, which is elements 124-127.
  //We will read the constants, update a few elements, and write the constants.
  //The entire memory area is read and written each time.

  //Note: writing and reading the cal area is an advanced operation. Please use caution
  //Read the cal constants.
  lngErrorcode := eGet(lngHandle, LJ_ioGET_CONFIG, LJ_chCAL_CONSTANTS, dummyDouble, @adblCalMem[0]);
  ErrorHandler(lngErrorcode, 0);
  //Display the last 4 elements.
  Writeln('Read Cal Constants [124-127] = ', adblCalMem[124], ', ', adblCalMem[125], ', ', adblCalMem[126], ', ', adblCalMem[127]);
  //Create 4 new pseudo-random numbers to write.  We will update the last
  //4 cal constants, but the rest will be unchanged.
  for i := 124 to 127 do
  begin
    adblCalMem[i] := Trunc(100*(Random(100)/100))-50;
  end;
  //The special value (0x4C6C) must be put in to write the cal constants.
  lngErrorcode := ePut(lngHandle, LJ_ioPUT_CONFIG, LJ_chCAL_CONSTANTS, 19564, @adblCalMem[0]);
  ErrorHandler(lngErrorcode, 0);
  Writeln('Write  Cal Constants [124-127] = ', adblCalMem[124], ', ', adblCalMem[125], ', ', adblCalMem[126], ', ', adblCalMem[127]);
  //Re-read the cal constants.
  lngErrorcode := eGet(lngHandle, LJ_ioGET_CONFIG, LJ_chCAL_CONSTANTS, dummyDouble, @adblCalMem[0]);
  ErrorHandler(lngErrorcode, 0);
  //Display the first 4 elements.
  Writeln('Read Cal Constants [124-127] = ', adblCalMem[124], ', ', adblCalMem[125], ', ', adblCalMem[126], ', ', adblCalMem[127]);
  }

  Readln;
end.
