//---------------------------------------------------------------------------
//
//  ue9rawio
//
//  Simple demonstration of using RAW_OUT and RAW_IN.  Sends a 0x70,0x70
//  which is a do nothing Comm command that simply responds with an echo.
//  Records the time for 1000 iterations and divides by 1000, to determine
//  the basic communication time of the LabJack.
//
//  For more in depth examples of low-level function calls, see the
//  Linux downloads.
//
//  support@labjack.com
//  Feb 3, 2006
//----------------------------------------------------------------------

program ue9rawio;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  Windows,
  Math,
  LJUDDelphi in '..\..\..\Wrapper\LJUDDelphi.pas';

  //This is our simple error handling procedure that is called after every UD
  //function call.  This function displays the errorcode and string description
  //of the error.  It also has an iteration input that is useful when processing
  //results in a loop (getfirst/getnext).
  procedure ErrorHandler(lngErrorcode: LJ_ERROR; lngIteration: longint);
  var
    err: array[0..254] of AnsiChar;
  begin
    if lngErrorcode <> LJE_NOERROR then
    begin
      ErrorToString(lngErrorcode, @err);
      Writeln('Error number = ' + IntToStr(lngErrorcode));
      Writeln('Error string = ' + string(err));
      Writeln('Iteration = ' + IntToStr(lngIteration));
      Writeln('');
      if lngErrorcode > LJE_MIN_GROUP_ERROR then
      begin
        //Quit if this is a group error.
        Readln;
        Halt;
      end;
    end;
  end;

var
  lngErrorcode: LJ_ERROR;
  lngHandle: LJ_HANDLE;
  time, i: longint;
  numIterations: longint;
  numBytesToWrite: double;
  numBytesToRead: double;
  writeArray: array[0..1] of byte;
  readArray: array[0..1] of byte;
  rate: double;

begin

  lngHandle := 0;
  time := 0;
  i := 0;
  numIterations := 1000;
  writeArray[0] := 112;
  writeArray[1] := 112;

  //Open the first found LabJack UE9.
  lngErrorcode := OpenLabJack(LJ_dtUE9, LJ_ctUSB, '1', 1, lngHandle);
  //lngErrorcode = OpenLabJack (LJ_dtUE9, LJ_ctETHERNET, '192.168.1.209', 0, lngHandle);
  ErrorHandler(lngErrorcode, 0);

  time := GetTickCount();

  for i := 0 to numIterations - 1 do
  begin
    numBytesToWrite := 2;
    numBytesToRead := 2;

    //Raw Out
    lngErrorcode := eGet(lngHandle, LJ_ioRAW_OUT, 0, numBytesToWrite, @writeArray[0]);
    ErrorHandler(lngErrorcode, 0);

    //Raw In
    lngErrorcode := eGet(lngHandle, LJ_ioRAW_IN, 0, numBytesToRead, @readArray[0]);
    ErrorHandler(lngErrorcode, 0);

  end;

  time := GetTickCount() - time;
  rate := time * 1.0 / numIterations;
  Writeln('Milleseconds per iteration = ', rate);

  Readln;
end.
