//---------------------------------------------------------------------------
//
//  ue9simple.dpr
//
//  Basic command/response UD UE9 example using Delphi.
//
//  support@labjack.com
//  March 27, 2006
//----------------------------------------------------------------------


program ue9simple;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  LJUDDelphi in '..\..\..\Wrapper\LJUDDelphi.pas';

  //This is our simple error handling procedure that is called after every UD
  //function call.  This function displays the errorcode and string description
  //of the error.  It also has an iteration input that is useful when processing
  //results in a loop (getfirst/getnext).
  procedure ErrorHandler(lngErrorcode: LJ_ERROR; lngIteration: longint);
  var
    err: array[0..254] of AnsiChar;
  begin
    if lngErrorcode <> LJE_NOERROR then
    begin
      ErrorToString(lngErrorcode, @err);
      Writeln('Error number = ' + IntToStr(lngErrorcode));
      Writeln('Error string = ' + string(err));
      Writeln('Iteration = ' + IntToStr(lngIteration));
      Writeln('');
      if lngErrorcode > LJE_MIN_GROUP_ERROR then
      begin
        //Quit if this is a group error.
        Readln;
        Halt;
      end;
    end;
  end;


var
  lngErrorcode: LJ_ERROR;
  dblDriverVersion: double;
  lngIOType, lngChannel: longint;
  dblValue: double;
  Value2, Value3: double;
  ValueDIBit, ValueDIPort, ValueCounter: double;
  lngHandle: LJ_HANDLE;
  lngGetNextIteration: longint;
  ch: char;
  pch: PChar;
  lngTemp: longint;
  dblTemp: double;
begin
  lngIOType := 0;
  lngChannel := 0;
  dblValue := 0;
  Value2 := 0;
  Value3 := 0;
  ValueDIBit := 0;
  ValueDIPort := 0;
  ValueCounter := 0;
  lngHandle := 0;
  lngTemp := 0;
  dblTemp := 0;

  //Read and display the UD version.
  dblDriverVersion := GetDriverVersion();
  Writeln('UD Driver Version = ' + FloatToStrF(dblDriverVersion, ffFixed, 4, 3));
  Writeln('');

  //Open the first found LabJack UE9.
  pch := '1';
  lngErrorcode := OpenLabJack(LJ_dtUE9, LJ_ctUSB, pch, 1, lngHandle);
  //pch := '192.168.1.209';
  //lngErrorcode := OpenLabJack (LJ_dtUE9, LJ_ctETHERNET, pch, 0, lngHandle);
  ErrorHandler(lngErrorcode, 0);

  //First some configuration commands.  These will be done with the ePut
  //function which combines the add/go/get into a single call.

  //Configure for 16-bit analog input measurements.
  lngErrorcode := ePut(lngHandle, LJ_ioPUT_CONFIG, LJ_chAIN_RESOLUTION, 16, 0);
  ErrorHandler(lngErrorcode, 0);

  //Configure the analog input range on channels 2 and 3 for bipolar gain=1.
  lngErrorcode := ePut(lngHandle, LJ_ioPUT_AIN_RANGE, 2, LJ_rgBIP5V, 0);
  ErrorHandler(lngErrorcode, 0);

  lngErrorcode := ePut(lngHandle, LJ_ioPUT_AIN_RANGE, 3, LJ_rgBIP5V, 0);
  ErrorHandler(lngErrorcode, 0);

  //Enable Counter0 which will appear on FIO0 (assuming no other
  //program has enabled any timers or Counter1).
  lngErrorcode := ePut(lngHandle, LJ_ioPUT_COUNTER_ENABLE, 0, 1, 0);
  ErrorHandler(lngErrorcode, 0);


  //Now we add requests to write and read I/O.  These requests
  //will be processed repeatedly by go/get statements in every
  //iteration of the while loop below.

  //Request AIN2 and AIN3.
  lngErrorcode := AddRequest(lngHandle, LJ_ioGET_AIN, 2, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  lngErrorcode := AddRequest(lngHandle, LJ_ioGET_AIN, 3, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Set DAC0 to 2.5 volts.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_DAC, 0, 2.5, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Read digital input FIO1.
  lngErrorcode := AddRequest(lngHandle, LJ_ioGET_DIGITAL_BIT, 1, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Set digital output FIO2 to output-high.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_DIGITAL_BIT, 2, 1, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Read digital inputs FIO3 through FIO7.
  lngErrorcode := AddRequest(lngHandle, LJ_ioGET_DIGITAL_PORT, 3, 0, 5, 0);
  ErrorHandler(lngErrorcode, 0);

  //Request the value of Counter0.
  lngErrorcode := AddRequest(lngHandle, LJ_ioGET_COUNTER, 0, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);


  while True do
  begin
    //Execute the requests.
    lngErrorcode := GoOne(lngHandle);
    ErrorHandler(lngErrorcode, 0);

    //Get all the results.  The input measurement results are stored.  All other
    //results are for configuration or output requests so we are just checking
    //whether there was an error.
    lngErrorcode := GetFirstResult(lngHandle, lngIOType, lngChannel, dblValue, lngTemp, dblTemp);
    ErrorHandler(lngErrorcode, 0);

    lngGetNextIteration := 0;  //Used by the error handling function.

    while lngErrorcode < LJE_MIN_GROUP_ERROR do
    begin
      if lngIOType = LJ_ioGET_AIN then
      begin
        if lngChannel = 2 then
          Value2 := dblValue;
        if lngChannel = 3 then
          Value3 := dblValue;
      end;

      if lngIOType = LJ_ioGET_DIGITAL_BIT then
        ValueDIBit := dblValue;
      if lngIOType = LJ_ioGET_DIGITAL_PORT then
        ValueDIPort := dblValue;
      if lngIOType = LJ_ioGET_COUNTER then
        ValueCounter := dblValue;

      lngErrorcode := GetNextResult(lngHandle, lngIOType, lngChannel, dblValue, lngTemp, dblTemp);
      if lngErrorcode <> LJE_NO_MORE_DATA_AVAILABLE then
        ErrorHandler(lngErrorcode, lngGetNextIteration);

      lngGetNextIteration := lngGetNextIteration + 1;

    end;

    Writeln('AIN2 = ' + FloatToStr(Value2));
    Writeln('AIN3 = ' + FloatToStr(Value3));
    Writeln('FIO1 = ' + FloatToStr(ValueDIBit));
    Writeln('FIO3-FIO7 = ' + FloatToStr(ValueDIPort));
    //Will read 31 if all 5 lines are pulled-high as normal.
    Writeln('Counter0 (FIO0) = ' + FloatToStr(ValueCounter));

    Writeln('');
    Writeln('Press Enter to go again or (q) and then Enter to quit');
    Read(ch);
    if ch = 'q' then
      Break;
  end;
end.
