//---------------------------------------------------------------------------
//
//  ue9simplestream.dpr
//
//  2-channel stream of AIN0 and FIOEIO using Delphi.
//
//  support@labjack.com
//  March 27, 2006
//----------------------------------------------------------------------

program ue9simplestream;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  conio,
  LJUDDelphi in '..\..\..\Wrapper\LJUDDelphi.pas';

  //This is our simple error handling procedure that is called after every UD
  //function call.  This function displays the errorcode and string description
  //of the error.  It also has an iteration input that is useful when processing
  //results in a loop (getfirst/getnext).
  procedure ErrorHandler(lngErrorcode: LJ_ERROR; lngIteration: longint);
  var
    err: array[0..254] of AnsiChar;
  begin
    if lngErrorcode <> LJE_NOERROR then
    begin
      ErrorToString(lngErrorcode, @err);
      Writeln('Error number = ' + IntToStr(lngErrorcode));
      Writeln('Error string = ' + string(err));
      Writeln('Iteration = ' + IntToStr(lngIteration));
      Writeln('');
      if lngErrorcode > LJE_MIN_GROUP_ERROR then
      begin
        //Quit if this is a group error.
        Readln;
        Halt;
      end;
    end;
  end;

var
  lngErrorcode: LJ_ERROR;
  lngHandle: LJ_HANDLE;
  lngGetNextIteration: longint;
  i, k: longint;
  lngIOType, lngChannel: longint;
  dblValue, dblCommBacklog: double;
  scanRate: double;
  delayms: longint;
  numScans: double;
  lngNumScans: longint;
  numScansRequested: double;
  adblData: array[0..3999] of double;  //Max buffer size (#channels*numScansRequested)
  lngTemp: longint;
  dblTemp: double;
begin
  lngHandle := 0;
  i := 0;
  k := 0;
  lngIOType := 0;
  lngChannel := 0;
  dblValue := 0;
  dblCommBacklog := 0;
  scanRate := 1000;
  delayms := 1000;
  numScans := 2000;  //2x the expected # of scans (2*scanRate*delayms/1000)
  lngNumScans := 0;
  lngTemp := 0;
  dblTemp := 0;


  //Open the first found LabJack UE9.
  lngErrorcode := OpenLabJack(LJ_dtUE9, LJ_ctUSB, '1', 1, lngHandle);
  //lngErrorcode = OpenLabJack (LJ_dtUE9, LJ_ctETHERNET, '192.168.1.209', 0, lngHandle);
  ErrorHandler(lngErrorcode, 0);


  //Configure the stream:
  //Configure all analog inputs for 12-bit resolution
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chAIN_RESOLUTION, 12, 0, 0);
  ErrorHandler(lngErrorcode, 0);
  //Configure the analog input range on channel 0 for bipolar +-5 volts.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_AIN_RANGE, 0, LJ_rgBIP5V, 0, 0);
  ErrorHandler(lngErrorcode, 0);
  //Set the scan rate.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chSTREAM_SCAN_FREQUENCY, scanRate, 0, 0);
  ErrorHandler(lngErrorcode, 0);
  //Give the driver a 5 second buffer (scanRate * 2 channels * 5 seconds).
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chSTREAM_BUFFER_SIZE, scanRate * 2 * 5, 0, 0);
  ErrorHandler(lngErrorcode, 0);
  //Configure reads to retrieve whatever data is available without waiting (wait mode LJ_swNONE).
  //See comments below to change this program to use LJ_swSLEEP mode.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chSTREAM_WAIT_MODE, LJ_swNONE, 0, 0);
  ErrorHandler(lngErrorcode, 0);
  //Define the scan list as AIN0 then FIOEIO.
  lngErrorcode := AddRequest(lngHandle, LJ_ioCLEAR_STREAM_CHANNELS, 0, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);
  lngErrorcode := AddRequest(lngHandle, LJ_ioADD_STREAM_CHANNEL, 0, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);
  lngErrorcode := AddRequest(lngHandle, LJ_ioADD_STREAM_CHANNEL, 193, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Execute the list of requests.
  lngErrorcode := GoOne(lngHandle);
  ErrorHandler(lngErrorcode, 0);

  //Get all the results just to check for errors.
  lngErrorcode := GetFirstResult(lngHandle, lngIOType, lngChannel, dblValue, lngTemp, dblTemp);
  ErrorHandler(lngErrorcode, 0);
  lngGetNextIteration := 0;  //Used by the error handling function.
  while lngErrorcode < LJE_MIN_GROUP_ERROR do
  begin
    lngErrorcode := GetNextResult(lngHandle, lngIOType, lngChannel, dblValue, lngTemp, dblTemp);
    if lngErrorcode <> LJE_NO_MORE_DATA_AVAILABLE then
      ErrorHandler(lngErrorcode, lngGetNextIteration);

    lngGetNextIteration := lngGetNextIteration + 1;
  end;

  //Start the stream.
  lngErrorcode := eGet(lngHandle, LJ_ioSTART_STREAM, 0, dblValue, 0);
  ErrorHandler(lngErrorcode, 0);

  //The actual scan rate is dependent on how the desired scan rate divides into
  //the LabJack clock.  The actual scan rate is returned in the value parameter
  //from the start stream command.
  Writeln('Actual Scan Rate = ' + FloatToStr(dblValue));
  Writeln('Actual Sample Rate = ' + FloatToStr(2 * dblValue));

  //Read data
  while _kbhit = 0 do  //Loop will run until any key is hit
  begin
    //Since we are using wait mode LJ_swNONE, we will wait a little, then
    //read however much data is available.  Thus this delay will control how
    //fast the program loops and how much data is read each loop.  An
    //alternative common method is to use wait mode LJ_swSLEEP where the
    //stream read waits for a certain number of scans.  In such a case
    //you would not have a delay here, since the stream read will actually
    //control how fast the program loops.

    //To change this program to use sleep mode,
    //  -change numScans to the actual number of scans desired per read,
    //  -change wait mode addrequest value to LJ_swSLEEP,
    //  -comment out the following Sleep command.

    Sleep(delayms);  //Remove if using LJ_swSLEEP.


    //init array so we can easily tell if it has changed
    for k := 0 to lngNumScans * 2 - 1 do
      adblData[k] := 9999.0;

    //Read the data.  We will request twice the number we expect, to
    //make sure we get everything that is available.
    //Note that the array we pass must be sized to hold enough SAMPLES, and
    //the Value we pass specifies the number of SCANS to read.
    numScansRequested := numScans;
    lngErrorcode := eGet(lngHandle, LJ_ioGET_STREAM_DATA, LJ_chALL_CHANNELS, numScansRequested, @adblData);
    //The displays the number of scans that were actually read.
    Writeln('');
    Writeln('Iteration # ' + IntToStr(i));
    Writeln('Number read = ' + FloatToStr(numScansRequested));
    //This displays just the first scan.
    Writeln('First scan = ' + FloatToStr(adblData[0]) + ', ' + FloatToStr(adblData[1]));
    ErrorHandler(lngErrorcode, 0);
    //Retrieve the current Comm backlog.  The UD driver retrieves stream data from
    //the UE9 in the background, but if the computer is too slow for some reason
    //the driver might not be able to read the data as fast as the UE9 is
    //acquiring it, and thus there will be data left over in the UE9 buffer.
    lngErrorcode := eGet(lngHandle, LJ_ioGET_CONFIG, LJ_chSTREAM_BACKLOG_COMM, dblCommBacklog, 0);
    Writeln('Comm Backlog = ' + FloatToStr(dblCommBacklog));
    i := i + 1;
  end;

  //Stop the stream
  lngErrorcode := eGet(lngHandle, LJ_ioSTOP_STREAM, 0, dblTemp, lngTemp);
  ErrorHandler(lngErrorcode, 0);

  Writeln('');
  Writeln('Done - Press Enter to exit');
  Readln;
end.
