//---------------------------------------------------------------------------
//
//  ue9timercounter
//
//  Basic UE9 example does a PWM output and a counter input, using AddGoGet method.
//
//  support@labjack.com
//  July 1, 2013
//----------------------------------------------------------------------


program timercounter;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  Windows,
  Math,
  LJUDDelphi in '..\..\..\Wrapper\LJUDDelphi.pas';

  //This is our simple error handling procedure that is called after every UD
  //function call.  This function displays the errorcode and string description
  //of the error.  It also has an iteration input that is useful when processing
  //results in a loop (getfirst/getnext).
  procedure ErrorHandler(lngErrorcode: LJ_ERROR; lngIteration: longint);
  var
    err: array[0..254] of AnsiChar;
  begin
    if lngErrorcode <> LJE_NOERROR then
    begin
      ErrorToString(lngErrorcode, @err);
      Writeln('Error number = ' + IntToStr(lngErrorcode));
      Writeln('Error string = ' + string(err));
      Writeln('Iteration = ' + IntToStr(lngIteration));
      Writeln('');
      if lngErrorcode > LJE_MIN_GROUP_ERROR then
      begin
        //Quit if this is a group error.
        Readln;
        Halt;
      end;
    end;
  end;

var
  lngErrorcode: LJ_ERROR;
  lngGetNextIteration: longint;
  lngIOType, lngChannel: longint;
  dblValue: double;
  lngHandle: LJ_HANDLE;
  dummyInt: longint;
  dummyDouble: double;

begin
  lngIOType := 0;
  lngChannel := 0;
  dblValue := 0;
  lngHandle := 0;

  //Open the first found LabJack UE9.
  lngErrorcode := OpenLabJack(LJ_dtUE9, LJ_ctUSB, '1', 1, lngHandle);
  ErrorHandler(lngErrorcode, 0);

  //Start by using the pin_configuration_reset IOType so that all
  //pin assignments are in the factory default condition.
  lngErrorcode := ePut(lngHandle, LJ_ioPIN_CONFIGURATION_RESET, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);


  //Disable all timers and counters to put everything in a known initial state.
  //Disable the timer and counter, and the FIO lines will return to digital I/O.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chNUMBER_TIMERS_ENABLED, 0, 0, 0);
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_COUNTER_ENABLE, 0, 0, 0, 0);
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_COUNTER_ENABLE, 1, 0, 0, 0);
  lngErrorcode := GoOne(lngHandle);

  //First we will output a square wave and count the number of pulses for about
  //1 second. Connect a jumper on the UE9 from FIO0 (PWM output) to
  //FIO1 (Counter0 input).

  //Use the fixed 750kHz timer clock source.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chTIMER_CLOCK_BASE, LJ_tc750KHZ, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Set the divisor to 3 so the actual timer clock is 250kHz.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chTIMER_CLOCK_DIVISOR, 3, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Enable 1 timer.  It will use FIO0.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chNUMBER_TIMERS_ENABLED, 1, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Configure Timer0 as 8-bit PWM.  Frequency will be 250k/256 = 977 Hz.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_TIMER_MODE, 0, LJ_tmPWM8, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Set the PWM duty cycle to 50%.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_TIMER_VALUE, 0, 32768, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Enable Counter0.  It will use FIO1 since 1 timer is enabled.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_COUNTER_ENABLE, 0, 1, 0, 0);
  ErrorHandler(lngErrorcode, 0);


  //Execute the requests on a single LabJack.  The driver will use a
  //single low-level TimerCounter command to handle all the requests above.
  lngErrorcode := GoOne(lngHandle);
  ErrorHandler(lngErrorcode, 0);


  //Get all the results just to check for errors.
  lngErrorcode := GetFirstResult(lngHandle, lngIOType, lngChannel, dblValue, dummyInt, dummyDouble);
  ErrorHandler(lngErrorcode, 0);
  lngGetNextIteration := 0;
  while lngErrorcode < LJE_MIN_GROUP_ERROR do
  begin
    lngErrorcode := GetNextResult(lngHandle, lngIOType, lngChannel, dblValue, dummyInt, dummyDouble);
    if not lngErrorcode = LJE_NO_MORE_DATA_AVAILABLE then
    begin
      ErrorHandler(lngErrorcode, lngGetNextIteration);
    end;

    lngGetNextIteration := lngGetNextIteration + 1;
  end;


  //Wait 1 second.
  Sleep(1000);

  //Request a read from the counter.
  lngErrorcode := eGet(lngHandle, LJ_ioGET_COUNTER, 0, dblValue, 0);
  ErrorHandler(lngErrorcode, 0);

  //This should read roughly 977 counts if FIO0 is shorted to FIO1.
  Writeln('Counter = ', dblValue);

  //Wait 1 second.
  Sleep(1000);

  //Request a read from the counter.
  lngErrorcode := eGet(lngHandle, LJ_ioGET_COUNTER, 0, dblValue, 0);
  ErrorHandler(lngErrorcode, 0);

  //This should read about 977 counts more than the previous read.
  Writeln('Counter = ', dblValue);


  //Disable the timer and counter, and the FIO lines will return to digital I/O.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chNUMBER_TIMERS_ENABLED, 0, 0, 0);
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_COUNTER_ENABLE, 0, 0, 0, 0);
  lngErrorcode := GoOne(lngHandle);

  readln;
end.
