//---------------------------------------------------------------------------
//
//  ue9efunctions.dpr
//
//  Demonstrates the UD E-functions with the LabJack UE9.
//
//  support@labjack.com
//  Feb 13, 2007
//----------------------------------------------------------------------

program u3efunctions;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  LJUDDelphi in '..\..\..\Wrapper\LJUDDelphi.pas';

  //This is our simple error handling procedure that is called after every UD
  //function call.  This function displays the errorcode and string description
  //of the error.  It also has an iteration input that is useful when processing
  //results in a loop (getfirst/getnext).
  procedure ErrorHandler(lngErrorcode: LJ_ERROR; lngIteration: longint);
  var
    err: array[0..254] of AnsiChar;
  begin
    if lngErrorcode <> LJE_NOERROR then
    begin
      ErrorToString(lngErrorcode, @err);
      Writeln('Error number = ' + IntToStr(lngErrorcode));
      Writeln('Error string = ' + string(err));
      Writeln('Iteration = ' + IntToStr(lngIteration));
      Writeln('');
      if lngErrorcode > LJE_MIN_GROUP_ERROR then
      begin
        //Quit if this is a group error.
        Readln;
        Halt;
      end;
    end;
  end;

var
  lngErrorcode: LJ_ERROR;
  dblValue: double;
  lngValue: longint;
  lngHandle: LJ_HANDLE;
  lngRange, lngResolution, lngBinary: longint;
  alngEnableTimers: array[0..5] of longint;
  alngEnableCounters: array[0..1] of longint;
  lngTimerClockBaseIndex, lngTimerClockDivisor: longint;
  alngTimerModes: array[0..5] of longint;
  adblTimerValues: array[0..5] of double;
  alngReadTimers, alngUpdateResetTimers: array[0..5] of longint;
  alngReadCounters, alngResetCounters: array[0..1] of longint;
  adblCounterValues: array[0..1] of double;
  highTime, lowTime, dutyCycle: double;
begin
  dblValue := 0;
  lngValue := 0;
  lngHandle := 0;
  adblCounterValues[0] := 0;
  adblCounterValues[1] := 0;

  //Open the first found LabJack UE9.
  lngErrorcode := OpenLabJack(LJ_dtUE9, LJ_ctUSB, '1', 1, lngHandle);
  //lngErrorcode = OpenLabJack (LJ_dtUE9, LJ_ctETHERNET, '192.168.1.209', 0, lngHandle);
  ErrorHandler(lngErrorcode, 0);

  //Take a measurement from AIN3.
  lngRange := LJ_rgBIP5V;
  lngResolution := 17;
  lngBinary := 0;
  lngErrorcode := eAIN(lngHandle, 3, 0, dblValue, lngRange, lngResolution, -1, lngBinary, 0, 0);
  ErrorHandler(lngErrorcode, 0);
  Writeln('AIN3 = ' + FloatToStr(dblValue));

  //Set DAC0 to 3.0 volts.
  dblValue := 3.0;
  lngBinary := 0;
  lngErrorcode := eDAC(lngHandle, 0, dblValue, lngBinary, 0, 0);
  ErrorHandler(lngErrorcode, 0);
  Writeln('DAC0 set to ' + FloatToStr(dblValue) + ' volts');

  //Read state of FIO2.
  lngErrorcode := eDI(lngHandle, 2, lngValue);
  ErrorHandler(lngErrorcode, 0);
  Writeln('FIO2 = ' + FloatToStr(lngValue));

  //Set the state of FIO3.
  lngValue := 0;
  lngErrorcode := eDO(lngHandle, 3, lngValue);
  ErrorHandler(lngErrorcode, 0);
  Writeln('FIO3 set to = ' + FloatToStr(lngValue));
  Writeln('');

  //Timers and Counters example.
  //First, a call to eTCConfig.  Fill the arrays with the desired values, then make the call.
  alngEnableTimers[0] := 1; //Enable Timer0 (uses FIO0).
  alngEnableTimers[1] := 1; //Enable Timer1 (uses FIO1).
  alngEnableTimers[2] := 1; //Enable Timer2 (uses FIO2).
  alngEnableTimers[3] := 1; //Enable Timer3 (uses FIO3).
  alngEnableTimers[4] := 0; //Disable Timer4.
  alngEnableTimers[5] := 0; //Disable Timer5.
  alngEnableCounters[0] := 1; //Enable Counter0 (uses FIO4).
  alngEnableCounters[1] := 1; //Enable Counter1 (uses FIO5).
  lngTimerClockBaseIndex := LJ_tc750KHZ;
  lngTimerClockDivisor := 3; //Thus timer clock is 250 kHz.
  alngTimerModes[0] := LJ_tmPWM8;
  //Timer0 is 8-bit PWM output.  Frequency is 250k/256 = 977 Hz.
  alngTimerModes[1] := LJ_tmDUTYCYCLE; //Timer1 is duty cyle input.
  alngTimerModes[2] := LJ_tmFIRMCOUNTER; //Timer2 is firmware counter input.
  alngTimerModes[3] := LJ_tmRISINGEDGES16; //Timer3 is 16-bit period measurement.
  alngTimerModes[4] := 0; //Timer4 not enabled.
  alngTimerModes[5] := 0; //Timer5 not enabled.
  adblTimerValues[0] := 16384; //Set PWM8 duty-cycle to 75%.
  adblTimerValues[1] := 0;
  adblTimerValues[2] := 0;
  adblTimerValues[3] := 0;
  adblTimerValues[4] := 0;
  adblTimerValues[5] := 0;
  lngErrorcode := eTCConfig(lngHandle, @alngEnableTimers, @alngEnableCounters, 0, lngTimerClockBaseIndex, lngTimerClockDivisor, @alngTimerModes, @adblTimerValues, 0, 0);

  ErrorHandler(lngErrorcode, 0);
  Writeln('Timers and Counters enabled.');
  Writeln('');

  Sleep(1000); //Wait 1 second.

  //Now, a call to eTCValues.
  alngReadTimers[0] := 0; //Don't read Timer0 (output timer).
  alngReadTimers[1] := 1; //Read Timer1;
  alngReadTimers[2] := 1; //Read Timer2;
  alngReadTimers[3] := 1; //Read Timer3;
  alngReadTimers[4] := 0; //Timer4 not enabled.
  alngReadTimers[5] := 0; //Timer5 not enabled.
  alngUpdateResetTimers[0] := 1; //Update Timer0;
  alngUpdateResetTimers[1] := 1; //Reset Timer1;
  alngUpdateResetTimers[2] := 1; //Reset Timer2;
  alngUpdateResetTimers[3] := 1; //Reset Timer3;
  alngUpdateResetTimers[4] := 0; //Timer4 not enabled.
  alngUpdateResetTimers[5] := 0; //Timer5 not enabled.
  alngReadCounters[0] := 1; //Read Counter0;
  alngReadCounters[1] := 1; //Read Counter1;
  alngResetCounters[0] := 1; //Reset Counter0.
  alngResetCounters[1] := 1; //Reset Counter1.
  adblTimerValues[0] := 32768; //Change Timer0 duty-cycle to 50%.
  adblTimerValues[1] := 0;
  adblTimerValues[2] := 0;
  adblTimerValues[3] := 0;
  adblTimerValues[4] := 0;
  adblTimerValues[5] := 0;
  lngErrorcode := eTCValues(lngHandle, @alngReadTimers, @alngUpdateResetTimers, @alngReadCounters, @alngResetCounters, @adblTimerValues, @adblCounterValues, 0, 0);
  ErrorHandler(lngErrorcode, 0);
  Writeln('Timer1 value = ' + FloatToStr(adblTimerValues[1]));
  Writeln('Timer2 value = ' + FloatToStr(adblTimerValues[2]));
  Writeln('Timer3 value = ' + FloatToStr(adblTimerValues[3]));
  Writeln('Counter0 value = ' + FloatToStr(adblCounterValues[0]));
  Writeln('Counter1 value = ' + FloatToStr(adblCounterValues[1]));

  //Convert Timer1 value to duty-cycle percentage.
  //High time is LSW
  highTime := trunc(adblTimerValues[1]) mod 65536;
  //Low time is MSW
  lowTime := trunc(adblTimerValues[1]) / 65536;
  //Calculate the duty cycle percentage.
  dutyCycle := 100 * highTime / (highTime + lowTime);
  Writeln('');
  Writeln('High clicks Timer1 = ' + FloatToStr(highTime));
  Writeln('Low clicks Timer1 = ' + FloatToStr(lowTime));
  Writeln('Duty cycle Timer1 = ' + FloatToStr(dutyCycle));


  //Disable all timers and counters.
  alngEnableTimers[0] := 0;
  alngEnableTimers[1] := 0;
  alngEnableTimers[2] := 0;
  alngEnableTimers[3] := 0;
  alngEnableTimers[4] := 0;
  alngEnableTimers[5] := 0;
  alngEnableCounters[0] := 0;
  alngEnableCounters[1] := 0;
  lngErrorcode := eTCConfig(lngHandle, @alngEnableTimers, @alngEnableCounters, 0, lngTimerClockBaseIndex, lngTimerClockDivisor, @alngTimerModes, @adblTimerValues, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  Writeln('');
  Writeln('Done - Press Enter to exit');
  Readln;
end.
